@@ -12,19 +12,28 @@ import zipfile
 from dataclasses import InitVar, dataclass, field
 from datetime import datetime
 from glob import glob
-from typing import IO, Literal
+from typing import IO, Any, Collection, Literal, Mapping, Sequence, Union, cast
 from pathlib import Path
 
+from warnings import warn
+
+from pandas.core.base import DataError
+
+from .rawquant_compat import _fdc_to_rawdata
+
+import matplotlib.pyplot as plt
+
 import pandas as pd
 import toml as toml
 from qslib.base import AccessLevel
 from qslib.plate_setup import PlateSetup
 
-from .data import df_from_readings, FilterDataReading
+from .data import FilterSet, df_from_readings, FilterDataReading
 from .machine import Machine, RunStatus
 from .tcprotocol import Protocol, Stage, Step
 from .util import *
 from ._version import version as __version__  # type: ignore
+from .normalization import Normalizer, NormRaw
 
 TEMPLATE_NAME = "ruo"
 
@@ -37,6 +46,8 @@ Specification-Title: Experiment Document Specification
 Specification-Version: 1.3.2
 """
 
+log = logging.getLogger(__name__)
+
 
 class AlreadyExistsError(ValueError):
     pass
@@ -92,8 +103,6 @@ def _find_or_raise(e: ET.ElementTree | ET.Element, n: str) -> ET.Element:
         return x
 
 
-log = logging.getLogger("experiment")
-
 _TABLE_FORMAT = {"markdown": "pipe", "org": "orgtbl"}
 
 
@@ -226,8 +235,7 @@ class Experiment:
     """
     _welldata: pd.DataFrame | None
 
-    filterdata: pd.DataFrame
-    temperatures: pd.DataFrame
+    temperatures: pd.DataFrame | None
     """
     A DataFrame of temperature readings, at one second resolution, during the experiment
     (and potentially slightly before and after, if included in the message log).
@@ -253,6 +261,10 @@ class Experiment:
 
     """
 
+    @property
+    def all_filters(self) -> Collection[FilterSet]:
+        return self.protocol.all_filters
+
     @property
     def welldata(self) -> pd.DataFrame:
         """
@@ -318,20 +330,25 @@ class Experiment:
         return s
 
     def info_html(self) -> str | ET.ElementTree | None:
-        summary = self.summary(plate='table')
+        summary = self.summary(plate="table")
 
         import matplotlib.pyplot as plt
+
         fig, ax = plt.subplots(figsize=(21.0 / 2.54, 15.0 / 2.54))
         self.protocol.tcplot(ax)
 
         o = io.StringIO()
-        fig.savefig(o, format='svg')
+        fig.savefig(o, format="svg")
         o = o.getvalue()
-        o = re.search("<svg.*</svg>", o, re.DOTALL)[0]
+        o = re.search("<svg.*</svg>", o, re.DOTALL)
+        if o is None:
+            raise Exception
+        o = o[0]
 
         import markdown
-        ost = markdown.markdown(summary, output_format='xhtml', extensions=['tables'])
-        
+
+        ost = markdown.markdown(summary, output_format="xhtml", extensions=["tables"])
+
         s = f"""
 <!DOCTYPE html>
 <html>
@@ -366,16 +383,35 @@ table, th, td {{
 
         return s
 
-        
-
     def __str__(self) -> str:
         return self.summary()
 
+    def _repr_markdown_(self) -> str:
+        if len(self.plate_setup.sample_wells) <= 12:
+            return self.summary()
+        else:
+            return self.summary(plate="table")
+
     @property
     def runtitle_safe(self) -> str:
         """Run name with " " replaced by "-"."""
         return self.name.replace(" ", "-")
 
+    @property
+    def rawdata(self) -> pd.DataFrame:
+        warn("rawdata is deprecated; use welldata instead")
+        if (self.activestarttime is None) or (self.welldata is None):
+            raise DataError
+        return _fdc_to_rawdata(
+            self.welldata,
+            self.activestarttime.timestamp(),
+        )
+
+    @property
+    def filterdata(self) -> pd.DataFrame:
+        warn("filterdata is deprecated; use welldata instead")
+        return self.rawdata
+
     def _ensure_machine(
         self, machine: Machine | str | None, password: str | None = None
     ) -> Machine:
@@ -565,7 +601,6 @@ table, th, td {{
             machine.abort_current_run()
 
     def get_status(self, machine: Machine | None = None) -> RunStatus:
-
         """
         Return the status of the experiment, if currently running.
 
@@ -579,41 +614,90 @@ table, th, td {{
         machine = self._ensure_machine(machine)
         return self._ensure_running(machine)
 
-    def sync_from_machine(self, machine: Machine | None = None) -> None:
+    def sync_from_machine(
+        self,
+        machine: Machine | None = None,
+        connect: bool = False,
+        log_method: Literal["copy", "eval"] = "eval",
+    ) -> None:
         """
         Try to synchronize the data in the experiment to the current state of the run on a
         machine, more efficiently than reloading everything.
         """
         machine = self._ensure_machine(machine)
 
-        # Get a list of all the files in the experiment folder
-        machine_files = machine.list_files(
-            f"experiments:{self.runtitle_safe}/", verbose=True, recursive=True
-        )
+        disconnect = False
+        if connect and not machine.connected:
+            disconnect = True
+            machine.connect()
 
-        # Transfer anything we don't have
-        for f in machine_files:
-            name = os.path.basename(f["path"])
-            sdspath = self._sdspath(re.sub(".*/apldbio/sds/(.*)$", r"\1", f["path"]))
-            logging.debug(f"checking {f['path']} mtime {f['mtime']} to {sdspath}")
-            if os.path.exists(sdspath) and os.path.getmtime(sdspath) >= float(
-                f["mtime"]
-            ):
-                logging.debug(f"{sdspath} has {os.path.getmtime(sdspath)}")
-                continue
-            from pathlib import Path  # FIXME
+        try:
+            # Get a list of all the files in the experiment folder
+            machine_files = machine.list_files(
+                f"experiments:{self.runtitle_safe}/", verbose=True, recursive=True
+            )
 
-            ldir = f["path"].split("/")[-2]
-            if ldir != "sds":
-                cp = Path(self._dir_eds) / ldir
-                if not cp.exists():
-                    cp.mkdir()
-            with open(sdspath, "wb") as b:
-                b.write(machine.read_file(f["path"]))
-            os.utime(sdspath, (f["atime"], f["mtime"]))
+            # Transfer anything we don't have
+            for f in machine_files:
+                name = os.path.basename(f["path"])
+                sdspath = self._sdspath(
+                    re.sub(".*/apldbio/sds/(.*)$", r"\1", f["path"])
+                )
+                logging.debug(f"checking {f['path']} mtime {f['mtime']} to {sdspath}")
+                if os.path.exists(sdspath) and os.path.getmtime(sdspath) >= float(
+                    f["mtime"]
+                ):
+                    logging.debug(f"{sdspath} has {os.path.getmtime(sdspath)}")
+                    continue
+                from pathlib import Path  # FIXME
 
-        # The message log is tricky. Ideally we'd use rsync or wc+tail. TODO
-        self._update_from_files()
+                ldir = f["path"].split("/")[-2]
+                if ldir != "sds":
+                    cp = Path(self._dir_eds) / ldir
+                    if not cp.exists():
+                        cp.mkdir()
+                if log_method == "copy" or (not f["path"].endswith("messages.log")):
+                    with open(sdspath, "wb") as b:
+                        b.write(machine.read_file(f["path"]))
+                elif log_method == "eval":
+                    with open(sdspath, "ab+") as b:
+
+                        # Seeking to the end of the file gives us the size
+                        curpos = b.seek(0, 1)
+
+                        # If we're less than 20 bytes, just copy the file:
+                        if curpos < 20:
+                            b.seek(0)
+                            b.write(machine.read_file(f["path"]))
+                        else:
+                            # Now seek 20 bytes back, and read those 20 bytes
+                            b.seek(-20, 1)
+                            checklocal = b.read(20)
+
+                            assert b.seek(0, 1) == curpos
+
+                            rdat = machine._get_log_from_byte(
+                                self.runtitle_safe, curpos - 20
+                            )
+
+                            if rdat[0:20] == checklocal:
+                                b.write(rdat[20:])
+                            else:
+                                log.error(
+                                    "Log comparison failure in eval method,"
+                                    f" falling back to copy. ('{checklocal}' ≠ '{rdat[0:20]}')"
+                                )
+                                b.seek(0)
+                                b.write(machine.read_file(f["path"]))
+                else:
+                    raise NotImplementedError
+                os.utime(sdspath, (f["atime"], f["mtime"]))
+
+            # The message log is tricky. Ideally we'd use rsync or wc+tail. TODO
+            self._update_from_files()
+        finally:
+            if disconnect:
+                machine.disconnect()
 
     def change_protocol(
         self, new_protocol: Protocol, machine: Machine | None = None
@@ -1537,7 +1621,9 @@ table, th, td {{
 
         self.name = exml.findtext("Name") or "unknown"
         self.user = _text_or_none(exml, "Operator")
-        self.createdtime = datetime.fromtimestamp(float(_find_or_raise(exml, "CreatedTime").text) / 1000.0)  # type: ignore
+        self.createdtime = datetime.fromtimestamp(
+            float(_find_or_raise(exml, "CreatedTime").text) / 1000.0
+        )  # type: ignore
         self.runstate = exml.findtext("RunState") or "UNKNOWN"  # type: ignore
         self.writesoftware = (
             exml.findtext(
@@ -1574,17 +1660,20 @@ table, th, td {{
             qstcxml = ET.parse(os.path.join(self._dir_eds, "qsl-tcprotocol.xml"))
 
             if (x := qstcxml.find("QSLibProtocolCommand")) is not None:
-                self._protocol_from_qslib = Protocol.from_command(x.text)
+                try:
+                    self._protocol_from_qslib = Protocol.from_command(x.text)
+                except ValueError:
+                    self._protocol_from_qslib = None
             else:
                 self._protocol_from_qslib = None
 
             if (x := qstcxml.findtext("MachineConnection")) and not self.machine:
                 self.machine = Machine(toml.loads(x))
-        # try:
-        self._protocol_from_xml = Protocol.from_xml(exml.getroot())
-        # except Exception as e:
-        #    print(e)
-        #    self._protocol_from_xml = None
+        try:
+            self._protocol_from_xml = Protocol.from_xml(exml.getroot())
+        except Exception as e:
+            print(e)
+            self._protocol_from_xml = None
 
     def _update_platesetup_xml(self) -> None:
         x = ET.parse(os.path.join(self._dir_eds, "plate_setup.xml"))
@@ -1620,10 +1709,6 @@ table, th, td {{
         else:
             self._welldata = None
 
-        if self._welldata is not None:
-            self.rawdata = _fdc_to_rawdata(self.welldata, self.activestarttime.timestamp() if self.activestarttime else None)  # type: ignore
-            self.filterdata = self.rawdata
-
     def data_for_sample(self, sample: str) -> pd.DataFrame:
         """Convenience function to return data for a specific sample.
 
@@ -1702,28 +1787,31 @@ table, th, td {{
 
         tt = []
 
-        # In normal runs, the protocol is there without the PROT command at the
-        # beginning of the log as an info command, with quote.message.  Let's
-        # try to grab it!
-        if m := re.match(
-            r"^Info (?:[\d.]+) (<quote.message>.*?</quote.message>)",
-            msglog,
-            re.DOTALL | re.MULTILINE,
-        ):
-            # We can get the prot name too, and sample volume!
-            rp = re.search(
-                r"NEXT RP (?:-CoverTemperature=(?P<ct>[\d.]+) )?(?:-SampleVolume=(?P<sv>[\d.]+) )?([\w-]+) (?P<protoname>[\w-]+)",
+        try:
+            # In normal runs, the protocol is there without the PROT command at the
+            # beginning of the log as an info command, with quote.message.  Let's
+            # try to grab it!
+            if m := re.match(
+                r"^Info (?:[\d.]+) (<quote.message>.*?</quote.message>)",
                 msglog,
-                re.IGNORECASE,
-            )
-            if rp:
-                prot = Protocol.from_command(f"PROT {rp['protoname']} {m[1]}")
-                if rp[1]:
-                    prot.volume = float(rp["sv"])
+                re.DOTALL | re.MULTILINE,
+            ):
+                # We can get the prot name too, and sample volume!
+                rp = re.search(
+                    r"NEXT RP (?:-CoverTemperature=(?P<ct>[\d.]+) )?(?:-SampleVolume=(?P<sv>[\d.]+) )?([\w-]+) (?P<protoname>[\w-]+)",
+                    msglog,
+                    re.IGNORECASE,
+                )
+                if rp:
+                    prot = Protocol.from_command(f"PROT {rp['protoname']} {m[1]}")
+                    if rp[1]:
+                        prot.volume = float(rp["sv"])
+                else:
+                    prot = Protocol.from_command(f"PROT unknown_name {m[1]}")
+                self._protocol_from_log = prot
             else:
-                prot = Protocol.from_command(f"PROT unknown_name {m[1]}")
-            self._protocol_from_log = prot
-        else:
+                self._protocol_from_log = None
+        except ValueError:
             self._protocol_from_log = None
 
         for m in re.finditer(
@@ -1746,7 +1834,7 @@ table, th, td {{
             self.temperatures = pd.DataFrame(
                 tt,
                 columns=pd.MultiIndex.from_tuples(
-                    [("time", "timestamp")]
+                    [(cast(str, "time"), cast(Union[str, int], "timestamp"))]
                     + [("sample", n) for n in range(1, self.num_zones + 1)]
                     + [("other", "heatsink"), ("other", "cover")]
                     + [("block", n) for n in range(1, self.num_zones + 1)]
@@ -1772,18 +1860,434 @@ table, th, td {{
             self.num_zones = None
             self.temperatures = None
 
+    def _find_anneal_stages(self) -> list[int]:
+        anneal_stages = []
+        for i, stage in enumerate(self.protocol.stages):
+            if any(
+                getattr(step, "collect", False)
+                and (getattr(step, "temp_increment", 0.0) < 0.0)
+                for step in stage.steps
+            ):
+                anneal_stages.append(i + 1)
+        return anneal_stages
 
-def _fdc_to_rawdata(fdc: pd.DataFrame, start_time: float) -> pd.DataFrame:
-    ret: pd.DataFrame = fdc.loc[:, (slice(None), "fl")].copy()
-    ret.columns = [f"{r}{c}" for r in "ABCDEFGH" for c in range(1, 13)]
-    for i in range(0, 6):
-        ret[f"temperature_{i+1}"] = fdc[f"A{(i+1)*2}", "rt"]
-    ret["temperature_avg"] = ret.loc[:, slice("temperature_1", "temperature_6")].mean(
-        axis=1
-    )
-    ret["timestamp"] = fdc["time", "timestamp"]
-    ret["exptime"] = ret["timestamp"] - start_time
-    ret["exphrs"] = (ret["timestamp"] - start_time) / 60 / 60
-    # ret['time'] = ret['time'].astype('datetime64[s]') # fixme: should we?
+    def _find_melt_stages(
+        self, anneal_stages: Sequence[int] | None
+    ) -> tuple[list[int], list[int]]:
+        melt_stages = []
+        for i, stage in enumerate(self.protocol.stages):
+            if any(
+                getattr(step, "collect", False)
+                and (getattr(step, "temp_increment", 0.0) > 0.0)
+                for step in stage.steps
+            ):
+                melt_stages.append(i + 1)
 
-    return ret
+        if (
+            (anneal_stages is not None)
+            and (len(anneal_stages) > 0)
+            and (len(melt_stages) > 0)
+            and (max(anneal_stages) + 1 < min(melt_stages))
+        ):
+            between_stages = [
+                x for x in range(max(anneal_stages) + 1, min(melt_stages))
+            ]
+        else:
+            between_stages = []
+
+        return melt_stages, between_stages
+
+    def plot_anneal_melt(
+        self,
+        samples: str | Sequence[str],
+        filters: str | FilterSet | Collection[str | FilterSet] | None = None,
+        anneal_stages: int | Sequence[int] | None = None,
+        melt_stages: int | Sequence[int] | None = None,
+        between_stages: int | Sequence[int] | None = None,
+        normalization: Normalizer = NormRaw(),
+        ax: plt.Axes | None = None,
+        marker: str | None = None,
+        legend: bool = True,
+        figure_kw: Mapping[str, Any] | None = None,
+        line_kw: Mapping[str, Any] | None = None,
+    ) -> plt.Axes:
+        """
+        Plots anneal/melt curves.
+
+        This uses solid lines for the anneal, dashed lines for the melt, and
+        dotted lines for anything "between" the anneal and melt (for example, a temperature
+        hold).
+
+        Line labels are intended to provide full information when read in combination with
+        the axes title.  They will only include information that does not apply to all
+        lines.  For example, if every line is from the same filter set, but different
+        samples, then only the sample will be shown.  If every line is from the same sample,
+        but different filter sets, then only the filter set will be shown. Wells are shown
+        if a sample has multiple wells.
+
+        samples
+            Either a reference to a single sample (a string), or a list of sample names.
+            Well names may also be included, in which case each well will be treated without
+            regard to the sample name that may refer to it.  Note this means you cannot
+            give your samples names that correspond with well references.
+
+        filters
+            Optional. A filterset (string or `FilterSet`) or list of filtersets to include in the
+            plot.  Multiple filtersets will be plotted *on the same axes*.  Optional;
+            if None, then all filtersets with data in the experiment will be included.
+
+        anneal_stages, melt_stages, between_stages: int | Sequence[int] | None
+            Optional. A stage or list of stages (integers, starting from 1), corresponding to the
+            anneal, melt, and stages between the anneal and melt (if any).  Any of these
+            may be None, in which case the function will try to determine the correct values
+            automatically.
+
+        normalization
+            Optional. A Normalizer instance to apply to the data.  By default, this is NormRaw, which
+            passes through raw fluorescence values.  NormToMeanPerWell also works well.
+
+        marker
+
+        ax
+            Optional.  An axes to put the plot on.  If not provided, the function will
+            create a new figure.
+
+        legend
+            Optional.  Determines whether a legend is included.
+
+        figure_kw
+            Optional.  A dictionary of options passed through as keyword options to
+            the figure creation.  Only applies if ax is None.
+
+        line_kw
+            Optional.  A dictionary of keywords passed to all three plotting commands.
+
+        """
+
+        if filters is None:
+            filters = self.all_filters
+
+        if isinstance(samples, str):
+            samples = [samples]
+
+        filters = _normalize_filters(filters)
+
+        if anneal_stages is None:
+            anneal_stages = self._find_anneal_stages()
+        elif isinstance(anneal_stages, int):
+            anneal_stages = [anneal_stages]
+        else:
+            anneal_stages = list(anneal_stages)
+
+        if melt_stages is None:
+            melt_stages, between_stages = self._find_melt_stages(anneal_stages)
+        elif isinstance(melt_stages, int):
+            melt_stages = [melt_stages]
+        else:
+            melt_stages = list(melt_stages)
+
+        if between_stages is None:
+            between_stages = []
+        elif isinstance(between_stages, int):
+            between_stages = [between_stages]
+        else:
+            between_stages = list(between_stages)
+
+        if ax is None:
+            ax = cast(
+                plt.Axes,
+                plt.figure(**({} if figure_kw is None else figure_kw)).add_subplot(),
+            )
+
+        data = normalization.normalize_scoped(self.welldata, "all")
+
+        all_wells = self.plate_setup.get_wells(samples)
+
+        reduceddata = data.loc[[f.lowerform for f in filters], all_wells]
+
+        reduceddata = normalization.normalize_scoped(reduceddata, "limited")
+
+        for filter in filters:
+            filterdat: pd.DataFrame = reduceddata.loc[filter.lowerform, :]  # type: ignore
+
+            annealdat: pd.DataFrame = filterdat.loc[anneal_stages, :]  # type: ignore
+            meltdat: pd.DataFrame = filterdat.loc[melt_stages, :]  # type: ignore
+
+            if len(between_stages) > 0:
+                betweendat: pd.DataFrame = filterdat.loc[between_stages, :]  # type: ignore
+
+            for sample in samples:
+                wells = self.plate_setup.get_wells(sample)
+
+                for well in wells:
+                    color = next(ax._get_lines.prop_cycler)["color"]
+
+                    if len(wells) > 1:
+                        label = f"{sample} ({well})"
+                    else:
+                        label = sample
+
+                    anneallines = ax.plot(
+                        annealdat.loc[:, (well, "st")],
+                        annealdat.loc[:, (well, "fl")],
+                        color=color,
+                        label=label,
+                        marker=marker,
+                        **(line_kw if line_kw is not None else {}),
+                    )
+
+                    meltlines = ax.plot(
+                        meltdat.loc[:, (well, "st")],
+                        meltdat.loc[:, (well, "fl")],
+                        color=color,
+                        linestyle="dashed",
+                        marker=marker,
+                        **(line_kw if line_kw is not None else {}),
+                    )
+
+                    if len(between_stages) > 0:
+                        betweenlines = ax.plot(
+                            betweendat.loc[:, (well, "st")],
+                            betweendat.loc[:, (well, "fl")],
+                            color=color,
+                            linestyle="dotted",
+                            marker=marker,
+                            **(line_kw if line_kw is not None else {}),
+                        )
+
+        ax.set_xlabel("temperature (°C)")
+
+        # FIXME: consider normalization
+        ax.set_ylabel("fluorescence")
+
+        if legend:
+            ax.legend()
+
+        ax.set_title(
+            _gen_axtitle(
+                self.name,
+                anneal_stages + between_stages + melt_stages,
+                samples,
+                wells,
+                filters,
+            )
+        )
+
+        return ax
+
+    def plot_over_time(
+        self,
+        samples: str | Sequence[str],
+        filters: str | FilterSet | Collection[str | FilterSet] | None = None,
+        stages: slice | int | Sequence[int] = slice(None),
+        normalization: Normalizer = NormRaw(),
+        ax: plt.Axes | Sequence[plt.Axes] | None = None,
+        legend: bool = True,
+        temperatures: Literal[False, "axes", "inset", "twin"] = False,
+        marker: str | None = None,
+        figure_kw: Mapping[str, Any] | None = None,
+        line_kw: Mapping[str, Any] | None = None,
+    ) -> Sequence[plt.Axes]:
+        """
+        Plots fluorescence over time, optionally with temperatures over time.
+
+        Line labels are intended to provide full information when read in combination with
+        the axes title.  They will only include information that does not apply to all
+        lines.  For example, if every line is from the same filter set, but different
+        samples, then only the sample will be shown.  If every line is from the same sample,
+        but different filter sets, then only the filter set will be shown. Wells are shown
+        if a sample has multiple wells.
+
+        samples
+            Either a reference to a single sample (a string), or a list of sample names.
+            Well names may also be included, in which case each well will be treated without
+            regard to the sample name that may refer to it.  Note this means you cannot
+            give your samples names that correspond with well references.
+
+        filters
+            Optional. A filterset (string or `FilterSet`) or list of filtersets to include in the
+            plot.  Multiple filtersets will be plotted *on the same axes*.  Optional;
+            if None, then all filtersets with data in the experiment will be included.
+
+        stages
+            Optional.  A stage, list of stages, or slice (all using integers starting
+            from 1), to include in the plot.  By default, all stages are plotted.
+            For example, to plot stage 2, use `stages=2`; to plot stages 2 and 4, use
+            `stages=[2, 4]`, to plot stages 3 through 15, use `stages=slice(3, 16)`
+            (Python ranges are exclusive on the end).  Note that is a slice, you
+            can use `None` instead of a number to denote the beginning/end.
+
+        normalization
+            Optional. A Normalizer instance to apply to the data.  By default, this is NormRaw, which
+            passes through raw fluorescence values.  NormToMeanPerWell also works well.
+
+        temperatures
+            Optional (default False).  Several alternatives for displaying temperatures.
+            "axes" uses a separate axes (created if ax is not provided, otherwise ax must
+            be a list of two axes).
+
+            Temperatures are from Experiment.temperature, and are thus the temperatures
+            as recorded during the run, not the set temperatures.  Note that this has a
+            *very* large number of data points, something that should be dealt with at
+            some point.
+
+        ax
+            Optional.  An axes to put the plot on.  If not provided, the function will
+            create a new figure.  If `temperatures="axes"`, however, you must provide
+            a list or tuple of *two* axes, the first for fluorescence, the second
+            for temperature.
+
+        legend
+            Optional.  Determines whether a legend is included.
+
+        figure_kw
+            Optional.  A dictionary of options passed through as keyword options to
+            the figure creation.  Only applies if ax is None.
+
+        line_kw
+            Optional.  A dictionary of keywords passed to fluorescence plot commands.
+
+        """
+
+        if filters is None:
+            filters = self.all_filters
+
+        filters = _normalize_filters(filters)
+
+        if isinstance(samples, str):
+            samples = [samples]
+
+        if isinstance(stages, int):
+            stages = []
+
+        fig = None
+
+        if ax is None:
+            if temperatures:
+                fig, ax = plt.subplots(
+                    2,
+                    1,
+                    sharex="all",
+                    gridspec_kw={"height_ratios": [3, 1]},
+                    **({} if figure_kw is None else figure_kw),
+                )
+            else:
+                fig, ax = plt.subplots(1, 1, **({} if figure_kw is None else figure_kw))
+                ax = [ax]
+
+        elif (not isinstance(ax, Sequence)) or isinstance(ax, plt.Axes):
+            ax = [ax]
+
+        ax = cast(Sequence[plt.Axes], ax)
+
+        data = normalization.normalize_scoped(self.welldata, "all")
+
+        all_wells = self.plate_setup.get_wells(samples) + ["time"]
+
+        reduceddata = data.loc[[f.lowerform for f in filters], all_wells]
+
+        reduceddata = normalization.normalize_scoped(reduceddata, "limited")
+
+        for filter in filters:
+            filterdat: pd.DataFrame = reduceddata.loc[filter.lowerform, :]  # type: ignore
+
+            for sample in samples:
+                wells = self.plate_setup.get_wells(sample)
+
+                for well in wells:
+                    color = next(ax[0]._get_lines.prop_cycler)["color"]
+
+                    label = _gen_label(sample, well, filter, samples, wells, filters)
+
+                    lines = ax[0].plot(
+                        filterdat.loc[stages, ("time", "hours")],
+                        filterdat.loc[stages, (well, "fl")],
+                        color=color,
+                        label=label,
+                        marker=marker,
+                        **(line_kw if line_kw is not None else {}),
+                    )
+
+        ax[-1].set_xlabel("time (hours)")
+
+        # FIXME: consider normalization
+        ax[0].set_ylabel("fluorescence")
+
+        if legend:
+            ax[0].legend()
+
+        if temperatures == "axes":
+            if len(ax) < 2:
+                raise ValueError("Temperature axes requires at least two axes in ax")
+
+            xlims = ax[0].get_xlim()
+
+            times = reduceddata.loc[(slice(None), stages), ("time", "hours")]
+            tmin, tmax = times.min(), times.max()
+
+            reltemps = self.temperatures.loc[
+                lambda x: (tmin <= x[("time", "hours")])
+                & (x[("time", "hours")] <= tmax),
+                :,
+            ]
+
+            for x in range(1, 7):
+                ax[1].plot(
+                    reltemps.loc[:, ("time", "hours")],
+                    reltemps.loc[:, ("sample", x)],
+                )
+
+            ax[0].set_xlim(xlims)
+            ax[1].set_xlim(xlims)
+
+            ax[1].set_ylabel("temperature (°C)")
+
+        ax[0].set_title(_gen_axtitle(self.name, stages, samples, wells, filters))
+
+        return ax
+
+    def plot_temperatures(self):
+        raise NotImplemented
+
+
+def _normalize_filters(
+    filters: str | FilterSet | Collection[str | FilterSet],
+) -> list[FilterSet]:
+    if isinstance(filters, str) or isinstance(filters, FilterSet):
+        filters = [filters]
+    return [FilterSet.fromstring(filter) for filter in filters]
+
+
+def _gen_label(sample, well, filter, samples, wells, filters) -> str:
+    label = ""
+    if len(samples) > 1:
+        label = str(sample)
+    if len(wells) > 1:
+        if len(label) > 0:
+            label += f" ({well})"
+        else:
+            label = str(well)
+    if len(filters) > 1:
+        if len(label) > 0:
+            label += f", {filter}"
+        else:
+            label = str(filter)
+
+    return label
+
+
+def _gen_axtitle(expname, stages, samples, wells, filters) -> str:
+    elems = []
+    if len(samples) == 1:
+        elems += samples
+    if len(filters) == 1:
+        elems += [str(f) for f in filters]
+
+    val = expname
+    if stages != slice(None):
+        val += f", stage {_pp_seqsliceint(stages)}"
+
+    if len(elems) > 0:
+        val += ": " + ", ".join(elems)
+
+    return val
