@@ -5,14 +5,15 @@ import logging
 import re
 import io
 from dataclasses import dataclass
-from typing import Any, Coroutine, Optional, Protocol
+from typing import Any, Coroutine, Literal, Optional, Protocol
+import time
 
 
 NL_OR_Q = re.compile(rb"(?:\n|<(/?)([\w.]+)[ *]*>)")
 Q_ONLY = re.compile(rb"<(/?)([\w.]+)[ *]*>")
 TIMESTAMP = re.compile(rb"(\d{8,}\.\d{3})")
 
-log = logging.getLogger("qsproto")
+log = logging.getLogger(__name__)
 
 
 class Error(Exception):
@@ -33,16 +34,20 @@ class ReplyError(IOError):
 class SubHandler(Protocol):
     def __call__(
         self, topic: bytes, message: bytes, timestamp: float | None = None
-    ) -> Coroutine[None, None, None]:
+    ) -> Coroutine[None, None, None]:  # pragma: no cover
         ...
 
 
 class QS_IS_Protocol(asyncio.Protocol):
+    lostconnection: Future[Any]
+    last_received: float
+
     def __init__(self) -> None:
         self.default_topic_handler = self._default_topic_handler
         self.readymsg = asyncio.get_running_loop().create_future()
         self.lostconnection = asyncio.get_running_loop().create_future()
         self.should_be_connected = False
+        self.last_received = time.time()
 
     def connection_lost(self, exc: Optional[Exception]) -> None:
         if self.should_be_connected:
@@ -52,6 +57,13 @@ class QS_IS_Protocol(asyncio.Protocol):
         self.lostconnection.set_result(exc)
         self.should_be_connected = False
 
+        # Cancel all futures; we'll never recover them.
+        for _, future in self.waiting_commands:
+            if future is not None:
+                future.cancel()
+
+        self.waiting_commands = []
+
     async def disconnect(self) -> None:
         self.should_be_connected = False
         await self.run_command("QUIT")
@@ -61,16 +73,27 @@ class QS_IS_Protocol(asyncio.Protocol):
         self.should_be_connected = True
         # setup connection.
         self.transport = transport
-        self.waiting_commands: list[tuple[bytes, None | Future[tuple[bytes, bytes]]]] = []
+        self.waiting_commands: list[
+            tuple[
+                bytes,
+                None
+                | Future[
+                    tuple[
+                        bytes, None | bytes, None | asyncio.Future[tuple[bytes, bytes]]
+                    ]
+                ],
+            ]
+        ] = []
         self.buffer = io.BytesIO()
         self.quote_stack: list[bytes] = []
         self.topic_handlers: dict[bytes, SubHandler] = {}
-        pass
+        self.last_received = time.time()
 
     async def _default_topic_handler(
         self, topic: bytes, message: bytes, timestamp: Optional[float] = None
     ) -> None:
         log.info(f"{topic.decode()} at {timestamp}: {message.decode()}")
+        self.last_received = time.time()
 
     async def handle_sub_message(self, message: bytes) -> None:
         i = message.index(b" ")
@@ -90,16 +113,25 @@ class QS_IS_Protocol(asyncio.Protocol):
         if ds.startswith((b"ERRor", b"OK", b"NEXT")):
             ms = ds.index(b" ")
             r = None
+            comfut_new = None
+            if ds.startswith(b"NEXT"):
+                loop = asyncio.get_running_loop()
+                comfut_new = loop.create_future()
             for i, (commref, comfut) in enumerate(self.waiting_commands):
                 if ds.startswith(commref, ms + 1):
                     if comfut is not None:
-                        comfut.set_result((ds[:ms], ds[ms + len(commref) + 2 :]))
+                        comfut.set_result(
+                            (ds[:ms], ds[ms + len(commref) + 2 :], comfut_new)
+                        )
                     else:
                         log.info(f"{commref!r} complete: {ds!r}")
                     r = i
                     break
             if r is None:
                 log.error(f"received unexpected command response: {ds!r}")
+            elif ds.startswith(b"NEXT"):
+                self.waiting_commands.append((self.waiting_commands[r][0], comfut_new))
+                del self.waiting_commands[r]
             else:
                 del self.waiting_commands[r]
         elif ds.startswith(b"MESSage"):
@@ -143,6 +175,7 @@ class QS_IS_Protocol(asyncio.Protocol):
                 self.buffer.write(data[lastwrite : m.end()])
                 lastwrite = m.end()
         self.buffer.write(data[lastwrite:])
+        self.last_received = time.time()
 
     async def run_command(
         self,
@@ -156,7 +189,7 @@ class QS_IS_Protocol(asyncio.Protocol):
         log.debug(f"Running command {comm.decode()}")
         loop = asyncio.get_running_loop()
 
-        comfut: Future[tuple[bytes, bytes]] = loop.create_future()
+        comfut = loop.create_future()
         if uid:
             import random
 
@@ -170,19 +203,22 @@ class QS_IS_Protocol(asyncio.Protocol):
             commref = comm
         self.waiting_commands.append((commref, comfut))
 
-        await asyncio.wait_for(asyncio.shield(comfut), ack_timeout)
-        state, msg = comfut.result()
+        try:
+            await asyncio.wait_for(asyncio.shield(comfut), ack_timeout)
+        except asyncio.CancelledError:
+            raise ConnectionError
+
+        state, msg, comnext = comfut.result()
         log.debug(f"Received ({state!r}, {msg!r})")
 
         if state == b"NEXT":
             if just_ack:
-                self.waiting_commands.append((commref, None))
-                return msg
+                # self.waiting_commands.append((commref, None))
+                return b""
             else:
-                comnext = loop.create_future()
-                self.waiting_commands.append((commref, comnext))
                 await comnext
-                state, msg = comnext.result()
+                state, msg, comnext2 = comnext.result()
+                assert comnext2 is None
                 log.debug(f"Received ({state!r}, {msg!r})")
 
         if state == b"OK":
@@ -191,7 +227,7 @@ class QS_IS_Protocol(asyncio.Protocol):
             raise CommandError(
                 comm.decode(), commref.decode(), msg.decode().rstrip()
             ) from None
-        else:
+        else:  # pragma: no cover
             raise CommandError(
                 comm.decode(), commref.decode(), (state + b" " + msg).decode()
             )
