@@ -6,12 +6,14 @@ from itertools import zip_longest
 from typing import (
     Any,
     ClassVar,
+    Collection,
     Iterable,
     List,
     Mapping,
     Optional,
     Sequence,
     Tuple,
+    Type,
     Union,
     cast,
 )
@@ -94,7 +96,7 @@ class ProtoCommand(ABC):
 class BaseStep(ABC):
     @property
     @abstractmethod
-    def body(self) -> Iterable[ProtoCommand]:
+    def body(self) -> List[ProtoCommand]:
         ...
 
     @property
@@ -107,16 +109,11 @@ class BaseStep(ABC):
     def repeat(self) -> int:
         ...
 
+    @abstractmethod
     def info_str(self, index=None, repeats=1) -> str:
-        if index is not None:
-            s = f"{index}. "
-        else:
-            s = f"- "
-        s += f"Step{' '+str(self.identifier) if self.identifier is not None else ''} of commands:\n"
-        s += "\n".join(f"  {i+1}. " + c.to_command() for i, c in enumerate(self.body))
-        return s
+        ...
 
-    def to_command(self, *, stepindex, **kwargs):
+    def to_command(self, *, stepindex: int, **kwargs) -> str:
         s = "STEP "
         if self.repeat != 1:
             s += f"-repeat={self.repeat} "
@@ -157,6 +154,32 @@ class Stage(XMLable):
             return False
         return self._steps == other._steps
 
+    @classmethod
+    def stepped_ramp(
+        cls: Type[Stage],
+        temp_from: float,
+        temp_to: float,
+        total_time: int,
+        nsteps: int,
+        collect: bool = False,
+        filters: Sequence = tuple(),
+    ):
+
+        step_time = int(round(total_time / nsteps))
+
+        temp_increment = round((temp_to - temp_from) / (nsteps - 1), 4)
+
+        return cls(
+            Step(
+                step_time,
+                temp_from,
+                collect=collect,
+                temp_increment=temp_increment,
+                filters=filters,
+            ),
+            repeat=nsteps,
+        )
+
     @property
     def steps(self) -> list[BaseStep]:
         return self._steps
@@ -325,13 +348,15 @@ class Stage(XMLable):
         ET.SubElement(e, "NumOfRepetitions").text = str(int(self.repeat))
         scycle: set[int] = set()
         for s in self._steps:
-            scycle.add(s.temp_incrementcycle)
-            scycle.add(s.time_incrementcycle)
-            assert isinstance(s, XMLable)
-            e.append(s.to_xml())
+            if isinstance(s, Step):
+                scycle.add(s.temp_incrementcycle)
+                scycle.add(s.time_incrementcycle)
+            if isinstance(s, XMLable):
+                e.append(s.to_xml())
         if len(scycle) > 1:
             log.warn("Approx")
-        ET.SubElement(e, "StartingCycle").text = str(next(iter(scycle)))
+        if scycle:
+            ET.SubElement(e, "StartingCycle").text = str(next(iter(scycle)))
         ET.SubElement(e, "AutoDeltaEnabled").text = "true"
 
         return e
@@ -470,7 +495,7 @@ class Protocol(XMLable):
 
         stage_start_time = 0.0
         stagenum = 1
-        previous_temperatures = [25.0]*6
+        previous_temperatures = [25.0] * 6
 
         for stage in self.stages:
             dataframe = stage.dataframe(stage_start_time, previous_temperatures)
@@ -508,6 +533,17 @@ class Protocol(XMLable):
         alltimes[1::2] = d["end_time"]
         return alltimes
 
+    @property
+    def all_filters(self) -> Collection[FilterSet]:
+        filters = {FilterSet.fromstring(f) for f in self.filters}
+
+        for stage in self.stages:
+            for step in stage.steps:
+                if isinstance(step, Step) and step.collect:
+                    filters |= {FilterSet.fromstring(f) for f in step.filters}
+
+        return filters
+
     def copy(self) -> Protocol:
         return deepcopy(self)
 
@@ -637,6 +673,9 @@ class Protocol(XMLable):
 
         return begin + "\n".join(stagestrs)
 
+    def _repr_markdown_(self) -> str:
+        return str(self)
+
     @classmethod
     def fromdict(cls, d: dict[str, Any]) -> "Protocol":
         s = [Stage(**x) for x in d["stages"]]
@@ -669,13 +708,12 @@ class Protocol(XMLable):
 @dataclass
 class Exposure(ProtoCommand):
     # We don't support persistent... it doesn't seem safe
-    settings: Mapping[FilterSet, Sequence[int]]
+    settings: List[Tuple[FilterSet, Sequence[int]]]
     state: str = "HoldAndCollect"
 
     def to_command(self, **kwargs) -> str:
         settingstrings = [
-            k.hacform + "," + ",".join(str(x) for x in v)
-            for k, v in self.settings.items()
+            k.hacform + "," + ",".join(str(x) for x in v) for k, v in self.settings
         ]
 
         return f"EXP -state={self.state} " + " ".join(settingstrings)
@@ -1036,3 +1074,30 @@ class Step(BaseStep, XMLable):
     @classmethod
     def fromdict(cls, d: dict[str, Any]) -> "Step":
         return cls(**d)
+
+
+@dataclass
+class CustomStep(BaseStep):
+    _body: List[ProtoCommand]
+    repeat: int = 1
+    identifier: int | str | None = None
+
+    @property
+    def body(self) -> List[ProtoCommand]:
+        return self._body
+
+    @body.setter
+    def body(self, v: Sequence[ProtoCommand]):
+        self._body = v
+
+    def total_duration(self, repeat: int = 1) -> int:
+        return 0
+
+    def info_str(self, index=None, repeats=1) -> str:
+        if index is not None:
+            s = f"{index}. "
+        else:
+            s = f"- "
+        s += f"Step{' '+str(self.identifier) if self.identifier is not None else ''} of commands:\n"
+        s += "\n".join(f"  {i+1}. " + c.to_command() for i, c in enumerate(self.body))
+        return s
